-- Misdirect Helper

local MDH = select(2, ...)

local MDH = LibStub("AceAddon-3.0"):NewAddon(MDH, "MDH", "AceConsole-3.0", "AceEvent-3.0")

local MACRO_TOOLTIP = "#showtooltip"
local MACRO_STOPCASTING = "\n/stopcasting"
local MACRO_FOCUS = "\n/use [@focus,help,nodead]"
local MACRO_PREFIX = "\n/use [@"
local MACRO_POSTFIX = ",nodead]"
local MACRO_PET = "\n/use [@pet,nodead]"
local TEXT_MDH = "Misdirect Helper"
local TEXT_ERROR = "Error"
local TEXT_ERROR_IN_COMBAT = "In combat, unable to create macro"
local TEXT_ERROR_TARGET_REQ = "Target required"
local TEXT_ERROR_FRIENDLY_PLAYER_REQ = "Target needs to be a friendly player"
local TEXT_ERROR_CLASS = "Incorrect Class, You should disable Misdirect Helper for this character"
local TEXT_NO_SLOTS_CHAR = "No free character macro slots, checking account macro slots"
local TEXT_NO_SLOTS_ACC = "No free account macro slots either, unable to add macro"
local TEXT_MPD = "Mythic+ Dungeon"
local TEXT_START = "Start"
local TEXT_DETECTED = "Detected"
local TEXT_CREATED = "Created"
local TEXT_UPDATED = "Updated"
local TEXT_AUTOMATIC = "Automatic"
local TEXT_MACRO_CHAR = "Specific Macro"
local TEXT_MACRO_ACC = "General Macro"
local TEXT_DROPDOWN = "Dropdown Menu"
local TEXT_DROPDOWN_DESC = "Select Assign Misdirect"
local TEXT_COMMANDLINE = "/mdh"
local TEXT_COMMANDLINE_DESC = "While targeting a friendly player"

local LINE_PARTY = 5
local LINE_RAID = 6

local UNIT_PLAYER = "player"
local UNIT_PLAYERTARGET = "playertarget"

local CLASS_HUNTER = 3
local CLASS_ROGUE = 4

local STRING_SPACE = " "

local MAX_ACC_MACROS = 120
local MAX_CHAR_MACROS = 18

local ROLE_TANK = "TANK"

local class_info = {
	[3] = {macroName = 'MDH_Hunter', spellName = GetSpellInfo(34477), iconName = 'ability_hunter_misdirection'},
	[4] = {macroName = 'MDH_Rogue', spellName = GetSpellInfo(57934), iconName = 'ability_rogue_tricksofthetrade'},
}

-- Challenge mode start event
local frame = CreateFrame("FRAME")
frame:RegisterEvent("CHALLENGE_MODE_START")

local function printHeader()
    print('|cFF33FF99Misdirect Helper|r: Usage')
end

local function printCommand(cmd, desc)
    print(('|cFF33FF99%s|r: %s'):format(cmd, desc))
end

local function GroupMembers(reversed, forceParty)
	local GROUP_PARTY = "party"
	local GROUP_RAID = "raid"
    local unit  = (not forceParty and IsInRaid()) and GROUP_RAID or GROUP_PARTY
    local numGroupMembers = forceParty and GetNumSubgroupMembers()  or GetNumGroupMembers()
	local i = reversed and numGroupMembers or (unit == GROUP_PARTY and 0 or 1)
	
    return function()
        local ret 
        if i == 0 and unit == GROUP_PARTY then 
            ret = UNIT_PLAYER
        elseif i <= numGroupMembers and i > 0 then
            ret = unit..i
        end
        i = i + (reversed and -1 or 1)
        return ret
    end
end

function MDH:UnitClass(unit)
	return select(3,UnitClass(unit))
end

function MDH:ClassCheck(unit)
	return MDH:UnitClass(unit) == CLASS_HUNTER or MDH:UnitClass(unit) == CLASS_ROGUE
end

function MDH:UnitName(unit)
	local name, realm = UnitName(unit)
	return name
end

	local target_name, target_realm = UnitName(UNIT_PLAYERTARGET)

function MDH:OnInitialize()
    MDH:RegisterChatCommand('mdh', 'HandleChatCommand')
end

function MDH:UpdateMacro(player_class, misdirect_target, is_mythic_plus_dungeon)

	local use_char_macros = true

	local macros_acc_total, macros_char_total = GetNumMacros()
	local macro_slot_assigned = GetMacroIndexByName(class_info[player_class].macroName)

	-- If macro not found
	if macro_slot_assigned == 0 then
		-- Check for free macro slots
		if macros_char_total == MAX_CHAR_MACROS then
			printHeader()
			printCommand(TEXT_MDH..STRING_SPACE..TEXT_ERROR, TEXT_NO_SLOTS_CHAR)
			
			if macros_acc_total == MAX_ACC_MACROS then
				printHeader()
				printCommand(TEXT_MDH..STRING_SPACE..TEXT_ERROR, TEXT_NO_SLOTS_ACC)
				return	
			else
				-- Flag to use account macros
				use_char_macros = false
			end
		end
	end

	-- Add new misdirect
	local macro_body_new = MACRO_TOOLTIP..STRING_SPACE..class_info[player_class].spellName..
					MACRO_STOPCASTING..
					MACRO_STOPCASTING..
					MACRO_FOCUS..class_info[player_class].spellName..
					MACRO_PREFIX..misdirect_target..MACRO_POSTFIX..class_info[player_class].spellName

	-- Add pet line for hunters
	if player_class == CLASS_HUNTER then
		macro_body_new = macro_body_new..MACRO_PET..class_info[player_class].spellName
	end

	-- Create macro if needed
	if InCombatLockdown() then
		printCommand(TEXT_MDH..STRING_SPACE..TEXT_ERROR, TEXT_ERROR_IN_COMBAT)
	else

		if is_mythic_plus_dungeon then
			printCommand(TEXT_MPD..STRING_SPACE..TEXT_START, TEXT_DETECTED)
		end

		if macro_slot_assigned == 0 then
			-- Create Macro
			CreateMacro(class_info[player_class].macroName, class_info[player_class].iconName, macro_body_new, use_char_macros, nil)

			printCommand(((use_char_macros and TEXT_MACRO_CHAR) or TEXT_MACRO_ACC)..STRING_SPACE..TEXT_CREATED,  class_info[player_class].macroName..' -> '..misdirect_target)
		else
			-- Edit Macro
			EditMacro(macro_slot_assigned, nil, nil, macro_body_new)

			printCommand(((use_char_macros and TEXT_MACRO_CHAR) or TEXT_MACRO_ACC)..STRING_SPACE..TEXT_UPDATED,  class_info[player_class].macroName..' -> '..misdirect_target)
		end
	end 
end

function MDH:HandleChatCommand(input)
	
	local args = {strsplit(' ', input)}
	
	-- Parse command line arguments
	if MDH:ClassCheck(UNIT_PLAYER) then
		
		local target_name = MDH:UnitName(UNIT_PLAYERTARGET)

		if args[1] == 'help' then
			printHeader()
			printCommand(TEXT_DROPDOWN, TEXT_DROPDOWN_DESC)
			printCommand(TEXT_COMMANDLINE, TEXT_COMMANDLINE_DESC)
			printCommand(TEXT_MPD, TEXT_AUTOMATIC)
		elseif target_name then	
			if target_name == MDH:UnitName(UNIT_PLAYER) then
				printCommand(TEXT_MDH..STRING_SPACE..TEXT_ERROR, TEXT_ERROR_FRIENDLY_PLAYER_REQ)
			else
				if UnitFactionGroup(UNIT_PLAYERTARGET) == UnitFactionGroup(UNIT_PLAYER) and UnitIsPlayer(UNIT_PLAYERTARGET) then
					MDH:UpdateMacro(MDH:UnitClass(UNIT_PLAYER), target_name, false)
				else
					printCommand(TEXT_MDH..STRING_SPACE..TEXT_ERROR, TEXT_ERROR_FRIENDLY_PLAYER_REQ)
				end
			end
		else
			printCommand(TEXT_MDH..STRING_SPACE..TEXT_ERROR, TEXT_ERROR_TARGET_REQ)
		end
	else
		printCommand(TEXT_MDH..STRING_SPACE..TEXT_ERROR, TEXT_ERROR_CLASS)
	end
end

-- Challenge mode start
frame:SetScript("OnEvent", function()

	if MDH:ClassCheck(UNIT_PLAYER) then
		for unit in GroupMembers() do
			local  role = UnitGroupRolesAssigned(unit)
			if role == ROLE_TANK then
			 	local target_name, _ = UnitName(unit)
			 
			 	MDH:UpdateMacro(MDH:UnitClass(UNIT_PLAYER), target_name, true)
				break;
			end   		
	 	end
	else
		printHeader()
		printCommand(TEXT_ERROR, TEXT_ERROR_CLASS)
	end    
end)

-- Add unit frame drop down for correct classes only
if MDH:ClassCheck(UNIT_PLAYER) then
	local dropdown = {}
	local unit_options
	local unit_name

	local validTypes = {
		FOCUS = true,
		PARTY = true,
		PLAYER = true,
		RAID = true,
		RAID_PLAYER = true,
		TARGET = true,
	}

	unit_options = {
		{
			text = "Assign Misdirect",
			func = function()
				if MDH:ClassCheck(UNIT_PLAYER) then 
					MDH:UpdateMacro(MDH:UnitClass(UNIT_PLAYER), unit_name, false)
				else
					printCommand(TEXT_MDH..STRING_SPACE..TEXT_ERROR, TEXT_ERROR_CLASS)
				end
			end
		}
	}

	local function IsValidDropDown(bdropdown)
		return  (type(bdropdown.which) == "string" and validTypes[bdropdown.which] ) 
			and	UnitFactionGroup(bdropdown.unit) == UnitFactionGroup(UNIT_PLAYER) 
			and UnitIsPlayer(bdropdown.unit)
	end

	local function OnToggle(bdropdown, event, options, level, data)
		if event == "OnShow" then
			
			if not IsValidDropDown(bdropdown) then
				return
			end

			unit_name = MDH:UnitName(bdropdown.unit)
					
			if not options[1] then
				for i = 1, #unit_options do
					options[i] = unit_options[i]
				end
				return true
			end
		elseif event == "OnHide" then
			if options[1] then
				for i = #options, 1, -1 do
					options[i] = nil
				end
				return true
			end
		end
	end

	local LibDropDownExtension = LibStub and LibStub:GetLibrary("LibDropDownExtension-1.0", true)

	LibDropDownExtension:RegisterEvent("OnShow OnHide", OnToggle, 1, dropdown)
end